@@ -1,9 +1,9 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 from decimal import Decimal
-from trytond.model import Workflow, ModelView, fields
-from trytond.pyson import Eval
+from trytond.model import Workflow, ModelView, fields, Check
 from trytond.pool import Pool, PoolMeta
+from trytond.transaction import Transaction
 
 __all__ = ['Move']
 __metaclass__ = PoolMeta
@@ -11,18 +11,17 @@ __metaclass__ = PoolMeta
 
 class Move:
     __name__ = 'stock.move'
-    fifo_quantity = fields.Float('FIFO Quantity',
-        digits=(16, Eval('unit_digits', 2)),
-        depends=['unit_digits'])
+    fifo_quantity = fields.Float('FIFO Quantity')
 
     @classmethod
     def __setup__(cls):
         super(Move, cls).__setup__()
+        t = cls.__table__()
         cls._sql_constraints += [
-                ('check_fifo_quantity_out',
-                    'CHECK(quantity >= fifo_quantity)',
-                    'FIFO quantity can not be greater than quantity.'),
-                ]
+            ('check_fifo_quantity_out',
+                Check(t, t.quantity >= t.fifo_quantity),
+                'FIFO quantity can not be greater than quantity.'),
+            ]
         cls._error_messages.update({
                 'del_move_fifo': ('You can not delete move "%s" that is used '
                     'for FIFO cost price.'),
@@ -38,8 +37,9 @@ class Move:
         fifo_quantity on the concerned incomming moves. Return the
         cost price for outputing the given product and quantity.
         '''
-
-        Uom = Pool().get('product.uom')
+        pool = Pool()
+        Uom = pool.get('product.uom')
+        Currency = pool.get('currency.currency')
 
         total_qty = Uom.compute_qty(self.uom, self.quantity,
             self.product.default_uom, round=False)
@@ -51,7 +51,11 @@ class Move:
         for move, move_qty in fifo_moves:
             consumed_qty += move_qty
 
-            move_unit_price = Uom.compute_price(move.uom, move.unit_price,
+            with Transaction().set_context(date=move.effective_date):
+                move_unit_price = Currency.compute(
+                    move.currency, move.unit_price,
+                    self.company.currency, round=False)
+            move_unit_price = Uom.compute_price(move.uom, move_unit_price,
                     move.product.default_uom)
             cost_price += move_unit_price * Decimal(str(move_qty))
 
@@ -89,7 +93,7 @@ class Move:
                 and self.to_location.type != 'storage'
                 and self.product.cost_price_method == 'fifo'):
             cost_price = self._update_fifo_out_product_cost_price()
-            if not self.cost_price:
+            if self.cost_price is None:
                 self.cost_price = cost_price
         super(Move, self)._do()
 
